/*
 *  pretty.c
 *  I/O Formatting Functions
 *  by Ben Allan
 *  Created: 01/98
 *  Version: $Revision: 1.3 $
 *  Version control file: $RCSfile: pretty.c,v $
 *  Date last modified: $Date: 98/06/16 15:47:43 $
 *  Last modified by: $Author: mthomas $
 *
 *  This file is part of the ASCEND Language Interpreter
 *
 *  Copyright (C) 1998 Carnegie Mellon University
 *
 *  The ASCEND Language Interpreter is free software; you can
 *  redistribute it and/or modify it under the terms of the GNU
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  The ASCEND Language Interpreter is distributed in hope that it
 *  will be useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139 USA.  Check
 *  the file named COPYING.  COPYING is found in ../compiler.
 */

#include <ctype.h>
#include "utilities/ascConfig.h"
#include "utilities/ascMalloc.h"
#include "general/pretty.h"

static void outstring(FILE *fp, char *s, int loop)
{
  while(loop>0) {
    FPUTC(' ',(fp));
    loop--;
  }
  FPRINTF(fp,"%s\n",s);
}

/*
 * remember not to change the content of the string, in the net.
 */
int print_long_string(FILE *fp,char *string, int width, int indent)
{
  char oldchar;
  char *head, *tail, *lastspace;
  int lw,count;

  /* there's an off by 1 in the logic, so ... */
  width++;
  if (fp == NULL || string == NULL) {
    return 0;
  }
  if (width < 4 || indent < 0) {
    return FPRINTF(fp,"%s",string);
  }
  lw = 0; count = 0;
  head = string;
  lastspace = NULL;

  while (*head != '\0') {
    tail = head;
    while (*tail != '\0' && lw < width) {
      if (isspace(*tail)) {
        lastspace = tail;
      }
      lw++;
      tail++;
    }
    if (lw < width) { /* found END of line */
      outstring(fp,head,indent);
      return count + lw + indent;
    }
    if (lastspace == NULL) { /* drat, line overrun */
      /* find next space or end */
      while (*tail != '\0' && !isspace(*tail)) {
        lw++;
        tail++;
      }
      if (*tail != '\0') { /* found a break */
        oldchar = *tail;
        *tail = '\0';
        outstring(fp,head,indent);
        lw = 0;
        count += (lw+indent);
        *tail = oldchar;
        head = tail;
      } else { /* found  END */
        outstring(fp,head,indent);
        return count + lw + indent;
      }
    } else { /*  found break */
      oldchar = *lastspace;
      *lastspace = '\0';
      outstring(fp,head,indent);
      lw = 0;
      count += (indent+(lastspace-head)+1);
      *lastspace = oldchar;
      head = lastspace;
      lastspace = NULL;
    }
    head++;
  }
  return 0;
}

#ifdef PLSTEST
int main()
{
  int w = 12;
  char *a = "123456789012345 67890123	45 678 90\n1234 56789012345678 9012";
  fprintf(stderr,">>>%s<<<\n",a);
  for (;w>=4;w--) {
    fprintf(stderr,"\n#fmt %d\n",w);
    print_long_string(stderr,a,w,4);
  }
  fprintf(stderr,">>>%s<<<\n",a);
}
#endif /* test */
